package client

import (
	"encoding/json"
	"fmt"
	"os"
)

// KafkaConf Kafka配置定义
type KafkaConf struct {
	// 客户端启动时，初始链接的服务端地址
	BootstrapServers string `json:"bootstrap.servers"`
	// 访问服务端使用的安全协议，当前只支持 PLAINTEXT 或 SASL_PLAINTEXT，根据访问地址选择，默认为 PLAINTEXT
	Protocol string `json:"security.protocol"`
	// 开启DEBUG模式
	Debug bool `json:"debug"`
	// 执行生产或者消费的Topic名称
	Topic string `json:"topic"`

	// 生产者配置
	Producer struct {
		// 生产可靠性配置，默认为1，支持配置0,1,-1(all)
		Acks string `json:"acks"`
		// 发送消息的最大批量大小，单位字节
		BatchSize string `json:"batch.size"`
	} `json:"producer"`

	// 消费者配置
	Consumer struct {
		// 消费者所属的消费组配置
		GroupId string `json:"group.id"`
		// 消费组无消费记录或者消费非法时，初始化的消费位置，支持earliest和latest两种配置，默认为latest
		AutoOffsetRest string `json:"auto.offset.reset"`
		// 是否开启自动提交，默认为true，开启自动提交
		AutoCommit bool `json:"enable.auto.commit"`
	} `json:"consumer"`

	// Sasl 认证相关配置
	Sasl struct {
		Enabled bool `json:"enabled"`
		// 认证算法，当前只支持 PLAIN 或 SCRAM-SHA-256，根据用户类型选择
		Mechanism string `json:"mechanism"`
		// 认证用户名
		UserName string `json:"username"`
		// 认证密码
		Password string `json:"password"`
	} `json:"sasl"`
}

func NewKafkaConf() *KafkaConf {
	conf := &KafkaConf{}

	conf.Protocol = "PLAINTEXT"
	conf.Debug = false
	conf.Producer.Acks = "1"
	conf.Producer.BatchSize = "16384"
	conf.Consumer.AutoOffsetRest = "earliest"
	conf.Consumer.AutoCommit = true
	conf.Sasl.Enabled = false

	return conf
}

func LoadConfig(configPath string) (*KafkaConf, error) {
	file, err := os.Open(configPath)
	if err != nil {
		return nil, fmt.Errorf("load configuration filed. %s", err.Error())
	}

	defer file.Close()

	config := NewKafkaConf()
	decoder := json.NewDecoder(file)

	err = decoder.Decode(config)
	if err != nil {
		return nil, fmt.Errorf("parse configuration failed. %s", err.Error())
	}

	err = config.validate()
	if err != nil {
		return nil, err
	}

	return config, nil
}

func (c *KafkaConf) validate() error {
	if c.Protocol != "PLAINTEXT" && c.Protocol != "SASL_PLAINTEXT" && c.Protocol != "SASL_SSL" {
		return fmt.Errorf("unsupported protocol(%s). only PLAINTEXT, SASL_PLAINTEXT and SASL_SSL are supported currently", c.Protocol)
	}

	if c.Protocol == "PLAINTEXT" {
		if c.Sasl.Enabled {
			return fmt.Errorf("sasl and SSL should be disabled if protocol(PLAINTEXT) is used")
		}

		return nil
	}

	if !c.Sasl.Enabled {
		return fmt.Errorf("sasl configuration shouble be provided")
	}

	if c.Sasl.Mechanism != "PLAIN" && c.Sasl.Mechanism != "SCRAM-SHA-256" {
		return fmt.Errorf("unsupported mechanism(%s). only PLAIN or SCRAM-SHA-256 is supported currently", c.Protocol)
	}

	return nil
}
