package client

import (
	"fmt"
	"github.com/confluentinc/confluent-kafka-go/kafka"
	"strconv"
)

func RunConsumer(config *KafkaConf) error {
	// Get consumer configuration object
	configMap := &kafka.ConfigMap{
		"bootstrap.servers":  config.BootstrapServers,
		"security.protocol":  config.Protocol,
		"group.id":           config.Consumer.GroupId,
		"auto.offset.reset":  config.Consumer.AutoOffsetRest,
		"enable.auto.commit": strconv.FormatBool(config.Consumer.AutoCommit),
	}

	if config.Debug {
		// Enable debug
		configMap.SetKey("debug", "ALL")
	}

	if config.Sasl.Enabled {
		// Set SASL configurations
		configMap.SetKey("sasl.mechanism", config.Sasl.Mechanism)
		configMap.SetKey("sasl.username", config.Sasl.UserName)
		configMap.SetKey("sasl.password", config.Sasl.Password)
	}

	// Create a new consumer
	consumer, err := kafka.NewConsumer(configMap)
	if err != nil {
		return err
	}

	// Subscribe topics
	err = consumer.SubscribeTopics([]string{config.Topic}, nil)
	if err != nil {
		return err
	}

	// Read 10 messages
	for count := 0; count < 10; {
		// Retrieve message from server. Poll method should be called by period.
		event := consumer.Poll(1000)
		switch msg := event.(type) {
		case *kafka.Message:
			// Handle one message.
			// NOTICE: message handler SHOULD NOT take too longer time that would make consumer failed.
			handleMessage(msg)
			count++
		}
	}

	if !config.Consumer.AutoCommit {
		// Commit consumer offsets manually.
		tps, err := consumer.Commit()
		if err != nil {
			fmt.Printf("Commit offsets failed. %s\n", err.Error())
		} else {
			fmt.Printf("Offset commited: \n")
			for _, tp := range tps {
				fmt.Printf("\t%s-%d: %d", *tp.Topic, tp.Partition, tp.Offset)
			}
		}
	}

	// Close consumer at last.
	return consumer.Close()
}

// Handle one message.
func handleMessage(msg *kafka.Message) {
	fmt.Printf("Consumed a message from topic %s [%d] at offset %d. key: (%s), value: (%s), timestamp: (%s) Headers: ",
		*msg.TopicPartition.Topic, msg.TopicPartition.Partition, msg.TopicPartition.Offset, msg.Key, msg.Value, msg.Timestamp.String())
	if msg.Headers != nil {
		fmt.Printf("(")
		for _, header := range msg.Headers {
			fmt.Printf("%s->%s,", header.Key, header.Value)
		}
		fmt.Printf(")")
	}
	fmt.Printf("\n")
}
