package client

import (
	"fmt"
	"github.com/confluentinc/confluent-kafka-go/kafka"
)

func RunProduce(config *KafkaConf) error {
	// Get producer configuration object
	configMap := &kafka.ConfigMap{
		"bootstrap.servers": config.BootstrapServers,
		"security.protocol": config.Protocol,
		"acks":              config.Producer.Acks,
		"batch.size":        config.Producer.BatchSize,
	}

	if config.Debug {
		// Enable debug
		configMap.SetKey("debug", "ALL")
	}

	if config.Sasl.Enabled {
		// Set SASL configurations
		configMap.SetKey("sasl.mechanism", config.Sasl.Mechanism)
		configMap.SetKey("sasl.username", config.Sasl.UserName)
		configMap.SetKey("sasl.password", config.Sasl.Password)
	}

	// Create a new producer
	producer, err := kafka.NewProducer(configMap)
	if err != nil {
		return err
	}

	// Handle sender results
	go callBack(producer)()

	// Get sender channel
	sendChannel := producer.ProduceChannel()

	// Send 10 messages
	for count := 0; count < 10; count++ {
		// Construct one message
		msg := &kafka.Message{
			// Topic-partition that message is send to
			TopicPartition: kafka.TopicPartition{
				// Topic name
				Topic: &config.Topic,
				// Partition number. A fixed partition or an auto-selected(kafka.PartitionAny) partition can be set
				Partition: kafka.PartitionAny,
			},
			// Message value. Can be a nil object
			Value: []byte(fmt.Sprintf("Bytedance test msg %d", count)),
			// Message key. Can be a nil object.
			// By default, messages will be sent to the same partition if theirs partitions are set to PartitionAny and keys are same.
			Key: []byte(fmt.Sprintf("Bytedance test key %d", count)),
			// Message attributes. Can be a nil object.
			Headers: []kafka.Header{
				{Key: "service", Value: []byte("kafka")},
				{Key: "version", Value: []byte("2.2.0")},
			},
		}

		// Send message to sender channel
		sendChannel <- msg
	}

	// Flush all messages to server before producer closing.
	producer.Flush(10000)
	// Close producer at last.
	producer.Close()
	return nil
}

// Callback method for sender results. This will never be called if Producer.Acks is equals with 0.
func callBack(p *kafka.Producer) func() {
	return func() {
		for event := range p.Events() {
			switch eType := event.(type) {
			case *kafka.Message:
				// Handler sender result
				if eType.TopicPartition.Error != nil {
					// Handle sender error
					fmt.Printf("Delivery failed: %v\n", eType.TopicPartition.Error)
				} else {
					// Handle sender success
					fmt.Printf("Delivered message to topic %s [%d] at offset %v\n",
						*eType.TopicPartition.Topic, eType.TopicPartition.Partition, eType.TopicPartition.Offset)
				}
			}
		}
	}
}
